//
//  MSPlacemarkSheetDetailsDataSource.m
//  MeridianSamples
//
//  Created by Cody Garvin on 8/30/17.
//  Copyright © 2017 Aruba Networks, Inc. All rights reserved.
//

#import <UIKit/UIKit.h>

#import "MSPlacemarkSheetDetailsDataSource.h"
#import "MSExampleHost.h"

#define LOAD_ERROR_MESSAGE NSLocalizedString(@"There was an error loading the destination", @"Error message when there is a network connection error")

CGFloat MRBottomSheetTableCellImageWidth = 24.0f;
CGFloat MRBottomSheetSeparatorMargin = 15.0f;

NSString *MRDDetailReusableIdentifer = @"MRDBottomSheetPlacemarkCell";
NSString *MRDDetailReusableSubtitleIdentifer = @"MRDBottomSheetPlacemarkSubtitleCell";


////////////////////////////////////////////////////////////////////////////////
#pragma mark - MSPlacemarkSheetDetailsDataSource Implementation
@interface MSPlacemarkSheetDetailsDataSource ()
@property (nonatomic, strong) NSError *error;
@property (nonatomic, weak) UIViewController *associatedViewController;
@property (nonatomic, strong) MRPlacemark *placemarkDetails;
@end

@implementation MSPlacemarkSheetDetailsDataSource

@dynamic callbackDelegate;
@dynamic heroImageContentMode;
@dynamic heroImage;
@dynamic bottomSheetTableData;

- (instancetype)initWithPlacemark:(MRPlacemark *)placemark viewController:(UIViewController *)viewController {
    if (self = [super init]) {
        self.associatedViewController = viewController;
        
        __weak typeof(self) weakSelf = self;
        [self grabPlacemarkDetailsForPlacemarkID:placemark.key.identifier completion:^(MRDPlacemarkDetails *details, NSError *error) {
            [weakSelf placemarkLoaded:details error:error];
        }];
        [self reloadSections];
    }
    return self;
}

- (void)executeActionForIndexPath:(NSIndexPath *)indexPath
               withViewController:(nullable UIViewController *)viewController {
    
    // Grab the data for the indexPath
    if ([self.bottomSheetTableData count] > indexPath.row) {
        [self.bottomSheetTableData objectAtIndex:indexPath.row];
    }
}

////////////////////////////////////////////////////////////////////////////////
#pragma mark - Hidden Methods
- (void)reloadSections {
    
    // Reload now that we have some reliable data
    // Let the delegate know we would like them to show data
    if (self.callbackDelegate &&
        [self.callbackDelegate respondsToSelector:@selector(data:loadedWithError:)]) {
        [self.callbackDelegate data:self.placemarkDetails loadedWithError:self.error];
    }
}

- (void)grabPlacemarkDetailsForPlacemarkID:(NSString *)placemarkID completion:(void (^)(MRDPlacemarkDetails *details, NSError *error))completion {
    
    if (!placemarkID || !MSExampleHost.appID) {
        if (completion) {
            completion(nil, [NSError errorWithDomain:@"Invalid Environment" code:-1001 userInfo:nil]);
        }
        return;
    }
    
    NSString *path = [NSString stringWithFormat:@"%@/api/locations/%@/placemarks/%@", [Meridian sharedConfig].editorURL, MSExampleHost.appID, placemarkID];
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc] init];
    [request setHTTPMethod:@"GET"];
    [request setURL:[NSURL URLWithString:path]];
    
    NSURLSession *session = [NSURLSession sessionWithConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    [[session dataTaskWithRequest:request completionHandler:^(NSData *data, NSURLResponse *response, NSError *error) {
        if (error) {
            NSLog(@"Request error %@", error.localizedDescription);
            return;
        }
        
        NSInteger statusCode = ((NSHTTPURLResponse *)response).statusCode;
        if (statusCode != 200) {
            NSLog(@"Error: HTTP status code %li", (long)statusCode);
            return;
        }
        
        if (!data) {
            NSLog(@"Error: did not receive detail data");
            return;
        }
        
        // Pull up the last map that tag was seen on
        NSDictionary *json = [NSJSONSerialization JSONObjectWithData:data options:0 error:nil];
        MRDPlacemarkDetails *details = [MRDPlacemarkDetails placemarkWithJSON:json app:[MREditorKey keyWithIdentifier:[MSExampleHost appID]]];
        if (completion) {
            dispatch_async(dispatch_get_main_queue(), ^{
                completion(details, nil);
            });
        }
        
    }] resume];
}

////////////////////////////////////////////////////////////////////////////////
#pragma mark - UITableViewDatasource && UITableViewDelegate Methods
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    UITableViewCell *returnCell = [super tableView:tableView cellForRowAtIndexPath:indexPath];
    
    if (!returnCell) {
        returnCell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:MRDDetailReusableIdentifer];
    }
    
    return returnCell;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    
    return [super tableView:tableView numberOfRowsInSection:section];
}

////////////////////////////////////////////////////////////////////////////////
- (void)placemarkLoaded:(MRDPlacemarkDetails *)placemarkDetails error:(NSError *)error {
   
    self.placemarkDetails = placemarkDetails;
    self.error = error;
    
    // Only reload if we have data that is valid
    if ([self placemarkDetailsHasImportantData:placemarkDetails]) {
        [self reloadSections];
    }
}

- (BOOL)placemarkDetailsHasImportantData:(MRDPlacemarkDetails *)details {
    return [details imageURL] != nil;
}

@end
