//
//  MSOverlayViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/22/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSOverlayViewController: MRMapViewController, MSExampleViewController {
    private var circleOverlay: MRPathOverlay?
    private var pathOverlay: MRPathOverlay?
    private var placemarkOverlays = [MREditorKey: MRPathOverlay]()

    private var blueOpaqueStroke = UIColor(red: 0, green: 0, blue: 1, alpha: 1)
    private var redOpaqueStroke = UIColor(red: 1, green: 0, blue: 0, alpha: 1)
    private var blueTransparentStroke = UIColor(red: 0, green: 0, blue: 0, alpha: 0.4)
    private var redTransparentStroke = UIColor(red: 1, green: 0, blue: 0, alpha: 0.4)
    private var blueTransparentFill = UIColor(red: 0, green: 0, blue: 1, alpha: 0.2)
    private var redTransparentFill = UIColor(red: 1, green: 0, blue: 0, alpha: 0.2)
    private var clearFill = UIColor(red: 0, green: 0, blue: 0, alpha: 0)
    
    convenience init() {
        self.init(nibName:nil, bundle:nil)
        mapView.mapKey = MREditorKey(forMap: MSExampleHost.mapID(), app: MSExampleHost.appID())
    }

    override func viewDidLoad() {
        super.viewDidLoad()

        let button0 = UIBarButtonItem(title: "Toggle Color", style: .plain, target: self, action: #selector(toggleColor))
        let button1 = UIBarButtonItem(title: "Toggle Overlay", style: .plain, target: self, action: #selector(toggleOverlay))
        navigationItem.rightBarButtonItems = [button0, button1]

        circleOverlay = nil
        pathOverlay = nil

        toggleOverlay()
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Custom Overlays"
    }

    static func exampleDetail() -> String {
        return """
        Example of how to add custom overlays.
        
        • Button to toggle the appearance of the overlays.
        • Button to toggle the existence of some overlays.
        """
    }

    // MARK: - MRMapViewDelegate

    override func mapView(_ mapView: MRMapView, rendererFor overlay: MRPathOverlay) -> MRPathRenderer? {
        let renderer = MRPathRenderer(overlay: overlay)
        renderer.lineWidth = 7
        renderer.lineCap = .butt
        renderer.lineJoin = .bevel

        // if the renderer just has the default colors, intialize them specific to this Sample
        if overlay == circleOverlay {
            renderer.lineWidth = 5
            if renderer.strokeColor != blueOpaqueStroke && renderer.strokeColor != redOpaqueStroke {
                renderer.strokeColor = blueOpaqueStroke
                renderer.fillColor = blueTransparentFill
            }
        } else if overlay == pathOverlay {
            renderer.lineWidth = 4
            if renderer.strokeColor != blueOpaqueStroke && renderer.strokeColor != redOpaqueStroke {
                renderer.strokeColor = blueOpaqueStroke
                renderer.fillColor = clearFill
            }
        } else {
            renderer.lineWidth = 3
            if renderer.strokeColor != blueTransparentStroke && renderer.strokeColor != redTransparentStroke {
                renderer.strokeColor = blueTransparentStroke
                renderer.fillColor = blueTransparentFill
            }
        }
        return renderer
    }

    override func mapView(_ mapView: MRMapView, viewFor annotation: MRAnnotation?) -> MRAnnotationView? {
        let superAnnotationView = super.mapView(mapView, viewFor: annotation)

        if annotation != nil && annotation is MRPlacemark {
            let placemark = annotation as! MRPlacemark
            if !placemark.area.isEmpty {

                // A new MRPathOverlay should only be created if it does not already exist
                if placemarkOverlays[placemark.key] == nil {
                    let overlay = MRPathOverlay(placemark: placemark)
                    overlay.userInteractionEnabled = true
                    mapView.add(overlay)
                    placemarkOverlays[placemark.key] = overlay
                    mapView.renderer(for: overlay)
                }
            }
        }
        return superAnnotationView
    }

    override func mapView(_ mapView: MRMapView, didTap overlay: MRPathOverlay) {
        super.mapView(mapView, didTap: overlay)
        let renderer: MRPathRenderer? = mapView.renderer(for: overlay)
        toggleStrokeColor(renderer!)
        toggleFillColor(renderer!)
    }

    // MARK: - Internal

    func toggleStrokeColor(_ renderer: MRPathRenderer) {
        if renderer.strokeColor == blueOpaqueStroke {
            renderer.strokeColor = redOpaqueStroke
        } else if renderer.strokeColor == redOpaqueStroke {
            renderer.strokeColor = blueOpaqueStroke
        } else if renderer.strokeColor == blueTransparentStroke {
            renderer.strokeColor = redTransparentStroke
        } else {
            renderer.strokeColor = blueTransparentStroke
        }
    }

    func toggleFillColor(_ renderer: MRPathRenderer) {
        if renderer.fillColor == clearFill {
            renderer.fillColor = clearFill
        } else if renderer.fillColor == redTransparentFill {
            renderer.fillColor = blueTransparentFill
        } else {
            renderer.fillColor = redTransparentFill
        }
    }

    @objc func toggleColor() {
        // let overlays = mapView.overlays
        var overlays = [MRPathOverlay] ()
        overlays = mapView.overlays()
        for overlay in overlays {
            let renderer: MRPathRenderer? = mapView.renderer(for: overlay)
            toggleStrokeColor(renderer!)
            toggleFillColor(renderer!)
        }
    }

    @objc func toggleOverlay() {
        if let overlay = circleOverlay {
            mapView.remove(overlay)
            circleOverlay = nil
        } else {
            circleOverlay = MRPathOverlay(circleOfCenter: CGPoint(x: 200, y: 200), radius: 50)
            circleOverlay?.userInteractionEnabled = true
            mapView.add(circleOverlay!)
        }

        if let overlay = pathOverlay {
            mapView.remove(overlay)
            pathOverlay = nil
        } else {
            let points = [ NSValue(cgPoint: CGPoint(x: 212, y: 298)),
                           NSValue(cgPoint: CGPoint(x: 370, y: 550)),
                           NSValue(cgPoint: CGPoint(x: 910, y: 240)),
                           NSValue(cgPoint: CGPoint(x: 1058, y: 270)) ]
            pathOverlay = MRPathOverlay(polylineOfPoints: points)
            pathOverlay?.userInteractionEnabled = true
            mapView.add(pathOverlay!)
        }
    }
}
