//
//  MSPlacemarkListViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/22/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSPlacemarkListTableViewController: UITableViewController, MSExampleViewController, UIPickerViewDelegate, UIPickerViewDataSource {
   
    private lazy var typePicker: UIPickerView = {
        let typePicker = UIPickerView()
        typePicker.dataSource = self
        typePicker.delegate = self
        return typePicker
    }()
    
    private lazy var namePicker: UIPickerView = {
        let namePicker = UIPickerView()
        namePicker.dataSource = self
        namePicker.delegate = self
        return namePicker
    }()
    
    private var request: MRPlacemarkRequest?
    private let Types = ["all", "generic", "restaurant", "restroom", "stairs", "exit"]
    private var typeFilter = ""
    private var typeButton: UIBarButtonItem?
    private let Names = ["all", "APPLE", "KFC", "Zopin"]
    private var nameFilter = ""
    private var nameButton: UIBarButtonItem?
    private var results = [MRPlacemarkResult]()

    override func viewDidLoad() {
        super.viewDidLoad()
        typeFilter = Types[0]
        nameFilter = Names[0]

        nameButton = UIBarButtonItem(title: "NameFilter", style: .plain, target: self, action: #selector(nameFilterAction))
        typeButton = UIBarButtonItem(title: "TypeFilter", style: .plain, target: self, action: #selector(typeFilterAction))
        navigationItem.rightBarButtonItems = [typeButton!, nameButton!]

        loadPlacemarks()
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Placemark List"
    }

    static func exampleDetail() -> String {
        return "Example of how to make requests for placemarks from Meridian servers."
    }

    // MARK: - UITableViewController

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return results.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let reuseIdentifier = "MRPlacemarkListCell"

        var cell: UITableViewCell
        if let dequeueCell = tableView.dequeueReusableCell(withIdentifier: reuseIdentifier) {
            cell = dequeueCell
        } else {
            cell = UITableViewCell(style: .value1, reuseIdentifier: reuseIdentifier)
        }

        if indexPath.row < results.count {
            let result = results[indexPath.row]
            if let placemarkName = result.placemark.name, placemarkName.count > 0 {
               cell.textLabel?.text = placemarkName
            } else if let typeName = result.placemark.typeName, typeName.count > 0 {
                cell.textLabel?.text = typeName
            } else {
                cell.textLabel?.text = "Placemark"
            }
            cell.accessoryType = .disclosureIndicator
        }
        return cell
    }

    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if indexPath.row < results.count {
            let result = results[indexPath.row]
            let controller = MRMapViewController()
            controller.pendingDestination = result.placemark
            navigationController?.pushViewController(controller, animated: true)
        }
    }

    // MARK: - UIPickerViewDelegate / UIPickerViewDatasource

    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }

    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        if(pickerView == typePicker ){
            return Types.count
        } else {
            return Names.count
        }
    }

    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        if(pickerView == typePicker ){
            if row < Types.count {
                return Types[row]
            }
        } else {
            if row < Names.count {
                return Names[row]
            }
        }
        return "array error..."
    }

    
    // MARK: - Action

    @objc private func typeFilterAction() {
       let controller = UIViewController()
        controller.title = "TypeFilter"
        controller.view.backgroundColor = UIColor.white
        controller.navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: .done, target: self, action: #selector(dismissTypeFilters))
        controller.view.addSubview(typePicker)

        let navController = UINavigationController.init(rootViewController: controller)
        present(navController, animated: true, completion: nil)
    }
    
    @objc private func dismissTypeFilters() {
        dismiss(animated: true, completion: { [weak self] in
            guard let strongSelf = self else {
                return
            }
            let selectedIndex = strongSelf.typePicker.selectedRow(inComponent: 0)
            if selectedIndex < strongSelf.Types.count {
                if strongSelf.typeFilter != strongSelf.Types[selectedIndex] {
                    strongSelf.typeFilter = strongSelf.Types[selectedIndex]
                    strongSelf.loadPlacemarks()
                }
            }
        })
    }

    @objc private func nameFilterAction() {
        let controller = UIViewController()
        controller.title = "NameFilter"
        controller.view.backgroundColor = UIColor.white
        controller.navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: .done, target: self, action: #selector(dismissNameFilters))
        controller.view.addSubview(namePicker)
        
        let navController = UINavigationController.init(rootViewController: controller)
        present(navController, animated: true, completion: nil)
    }
    
    @objc private func dismissNameFilters() {
        dismiss(animated: true, completion: { [weak self] in
            guard let strongSelf = self else {
                return
            }
            let selectedIndex = strongSelf.namePicker.selectedRow(inComponent: 0)
            if selectedIndex < strongSelf.Names.count {
                if strongSelf.nameFilter != strongSelf.Names[selectedIndex] {
                    strongSelf.nameFilter = strongSelf.Names[selectedIndex]
                    strongSelf.loadPlacemarks()
                }
            }
        })
    }

    // MARK: - Utilities

    private func loadPlacemarks() {
        refreshControl?.beginRefreshing()
        typeButton?.isEnabled = false
        nameButton?.isEnabled = false

        results.removeAll()
        request = MRPlacemarkRequest()
        request?.app = MREditorKey(identifier: MSExampleHost.appID())

        let typeIndex = Types.index(of: typeFilter)
        let nameIndex = Names.index(of: nameFilter)
        
        if typeIndex! > 0, nameIndex! > 0 {
            request?.filters = [MRQueryFilter(field: "type", value: typeFilter), MRQueryFilter(field: "name", value: nameFilter)]
        } else if typeIndex! > 0 {
            request?.filters = [MRQueryFilter(field: "type", value: typeFilter)]
        } else if nameIndex! > 0 {
            request?.filters = [MRQueryFilter(field: "name", value: nameFilter)]
        }

        request?.start(completionHandler: { [weak self] response, error in
            self?.placemarksLoadedWithResposne(response: response, error: error)
        })
    }

    private func placemarksLoadedWithResposne(response: MRPlacemarkResponse?, error: Error?) {
        if let requestError = error {
            print("Error loading placemarks: " + requestError.localizedDescription)
            return
        }

        if let responseResults = response?.results {
            results += responseResults
            tableView.reloadData()
        }

        if let nextPage = response?.nextPage {
            nextPage.start(completionHandler: { [weak self] nextResponse, nextError in
                self?.placemarksLoadedWithResposne(response: nextResponse, error: nextError)
            })
        } else {
            refreshControl?.endRefreshing()
            typeButton?.isEnabled = true
            nameButton?.isEnabled = true
        }
    }
}
